import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { BackTop, notification } from 'antd'
import Api from '@/api'
import DataSearch from './modules/search'
import DataAction from './modules/action'
import DataTable from './modules/table'
import Loading from '@/components/loading'
import zhCN from '@/locales/zh-CN/main.js'
import enUS from '@/locales/en-US/main.js'
import './index.scss'

export default class DataManage extends Component {
  static propTpyes = {
    MenuNo: PropTypes.string // 标签页数组
  }

  state = {
    dict: sessionStorage.getItem('lang') !== 'en-US' ? zhCN : enUS, // 数据字典
    searchlist: null, // 搜索条件
    searchparam: null, // 搜索参数
    actions: null, // 按钮
    columns: null, // 显示列
    selectable: false, // 表格是否可选
    datafunc: null, // 获取数据及显示列的存储过程名称
    data: null,
    total: 0,
    loading: true,
    param: {
      pageIndex: 1,
      pageSize: 10,
      orderColumn: '',
      orderType: '',
      search: ''
    }
  }

  async loadconfig () {
    let param = {
      func: 's_Get_Page_wheproc',
      MenuNo: this.props.MenuNo
    }
    let result = await Api.commonInterface(param)
    if (result.status) {
      let newconfig = {}
      if (result.sWheredata && result.sWheredata.length > 0) {
        newconfig.searchlist = result.sWheredata
      }
      if (result.sMenusdata && result.sMenusdata.length > 0) {
        newconfig.actions = result.sMenusdata
      }
      newconfig.selectable = result.CheckBox === 'true'
      newconfig.datafunc = result.sWhereAction
      this.setState(newconfig)
    } else {
      notification.warning({
        top: 92,
        message: result.message
      })
    }
  }

  async loadDbdata (searches, pageIndex = 1, pageSize = 10, orderColumn = '', orderType = '') {
    let param = {
      func: this.state.datafunc,
      MenuNo: this.props.MenuNo,
      PageSize: pageSize,
      PageIndex: pageIndex,
      SortName: orderColumn,
      SortOrder: orderType,
      ...searches
    }
    let result = await Api.commonInterface(param)
    if (result.status) {
      result.sGriddata.length = 4
      this.setState({
        columns: result.sGriddata,
        data: result.data.map((item, index) => {
          item.key = index
          return item
        }),
        total: result.Total,
        searchparam: searches,
        loading: false
      })
    } else {
      notification.warning({
        top: 92,
        message: result.message
      })
      this.setState({
        searchparam: searches,
        loading: false
      })
    }
  }

  refreshbysearch = (searches, type) => {
    // 搜索条件变化
    if (type === 'reset') {
      this.setState({
        columns: null,
        data: null,
        total: 0
      })
    } else {
      this.loadDbdata(searches)
      this.setState({
        loading: true,
        columns: null
      })
    }
  }

  refreshbytable = (pagination, filters, sorter) => {
    // 表格查询条件修改
    if (sorter.order) {
      let _chg = {
        ascend: 'asc',
        descend: 'desc'
      }
      sorter.order = _chg[sorter.order]
    }
    this.loadDbdata(this.state.searchparam, pagination.current, pagination.pageSize, sorter.field, sorter.order)
    this.setState({
      loading: true
    })
  }

  refreshbyaction = () => {
    // 按钮操作后刷新表格,重置页码及选择项
    this.refs.dataTable.resetTable()
    this.loadDbdata(this.state.searchparam)
    this.setState({
      loading: true
    })
  }

  gettableselected = () => {
    // 获取表格选择项
    let data = []
    this.refs.dataTable.state.selectedRowKeys.forEach(item => {
      data.push(this.refs.dataTable.props.data[item])
    })
    return data
  }

  UNSAFE_componentWillMount () {
    // 组件加载时，获取菜单数据
    this.loadconfig()
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props), fromJS(nextProps)) || !is(fromJS(this.state), fromJS(nextState))
  }

  render() {
    return (
      <div className="datamanage">
        {!this.state.searchlist && <Loading />}
        {this.state.searchlist &&
          <DataSearch
            refreshdata={this.refreshbysearch}
            searchlist={this.state.searchlist}
            dict={this.state.dict}
          />
        }
        {this.state.actions &&
          <DataAction
            MenuNo={this.props.MenuNo}
            refreshdata={this.refreshbyaction}
            gettableselected={this.gettableselected}
            actions={this.state.actions}
            dict={this.state.dict}
          />
        }
        {this.state.columns &&
          <DataTable
            ref="dataTable"
            MenuNo={this.props.MenuNo}
            refreshdata={this.refreshbytable}
            columns={this.state.columns}
            data={this.state.data}
            selectable={this.state.selectable}
            total={this.state.total}
            loading={this.state.loading}
            dict={this.state.dict}
          />
        }
        <BackTop>
          <div className="ant-back-top">
            <div className="ant-back-top-content">
              <div className="ant-back-top-icon"></div>
            </div>
          </div>
        </BackTop>
      </div>
    )
  }
}